"""
Browser info for Browser Stack api

"""

win_7_pc_chrome = {"os":"Windows",
    "os_version":"7",
    "browser":"chrome",
    "device":None,
    "browser_version":"50.0",
    "real_mobile":None}

win_7_pc_ie = {"os":"Windows",
    "os_version":"7",
    "browser":"ie",
    "browser_version":"9.0"}

win_10_pc_edge = {"os":"Windows",
    "os_version":"10",
    "browser":"edge",
    "device":None,
    "browser_version":"15.0",
    "real_mobile":None}

win_7_pc_firefox = {"os":"Windows",
    "os_version":"7",
    "browser":"firefox",
    "device":None,
    "browser_version":"45.0",
    "real_mobile":None}

osx_yosemite_pc_safari = {"os":"OS X",
    "os_version":"El Capitan",
    "browser":"safari",
    "device":None,
    "browser_version":"9.1",
    "real_mobile":None}

android = {"os":"android",
    "os_version":"5.0",
    "browser":"Android Browser",
    "device":"Google Nexus 5",
    "browser_version":None,
    "real_mobile":None}


win_10_pc_firefox = {"os":"Windows",
    "os_version":"10",
    "browser":"firefox",
    "device": None,
    "browser_version":"62.0",
    "real_mobile": None}


win_10_pc_chrome = {"os":"Windows",
    "os_version":"10",
    "browser":"chrome",
    "device": None,
    "browser_version":"69.0",
    "real_mobile": None}

win_10_pc_edge = {"os":"Windows",
    "os_version":"10",
    "browser":"edge",
    "device": None,
    "browser_version":"17.0",
    "real_mobile": None}

osx_high_sierra_safari = {'os_version': 'High Sierra',
    'os': 'OS X',
    'browser': 'safari',
    'device': None,
    'browser_version': '11.1',
    'real_mobile': None,}

all_browsers = {'browsers': [
    win_7_pc_chrome,
    win_7_pc_ie,
    win_10_pc_edge,
    win_7_pc_firefox,
    osx_yosemite_pc_safari,
    android,
]}

osx_browser = {'browsers': [osx_yosemite_pc_safari]}

gdi_browsers = {'browsers': [win_7_pc_ie]}

android_browsers = {'browsers': [android]}

vf_browsers = {'browsers': [
    win_10_pc_firefox,
    win_10_pc_chrome,
    osx_high_sierra_safari
]}

safari_latest = {'browsers': [osx_high_sierra_safari]}

test_browsers = {
    'all_browsers': all_browsers,
    'gdi_browsers': gdi_browsers,
    'osx_browser': osx_browser,
    'android_browsers': android_browsers,
    'vf_browsers': vf_browsers,
    'safari_latest': safari_latest,
}



"""
Hardcore tests for families with manual hinting. This should also be run for
popular families.

"""
from __future__ import print_function, division, absolute_import, unicode_literals
import argparse
import os
import time
import logging
from diffbrowsers.diffbrowsers import DiffBrowsers
from diffbrowsers.utils import load_browserstack_credentials, cli_reporter
from diffbrowsers.browsers import test_browsers

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    
    parser.add_argument('fonts_after', nargs="+", help="Fonts after paths")
    before_group = parser.add_argument_group(title="Fonts before input")
    before_input_group = before_group.add_mutually_exclusive_group(required=True)
    before_input_group.add_argument('-fb', '--fonts-before', nargs="+",
                                  help="Fonts before paths")
    before_input_group.add_argument('-gf', '--from-googlefonts', action='store_true',
                               help="Diff against GoogleFonts instead of fonts_before")
    parser.add_argument('-o', '--output-dir', help="Directory for output images",
                        required=True)

    args = parser.parse_args()
    auth = load_browserstack_credentials()

    browsers_to_test = test_browsers['all_browsers']

    fonts_before = 'from-googlefonts' if args.from_googlefonts \
                    else args.fonts_before

    diffbrowsers = DiffBrowsers(dst_dir=args.output_dir, browsers=browsers_to_test)
    diffbrowsers.new_session(fonts_before, args.fonts_after)

    for pt in [7, 12, 24]:
        diffbrowsers.diff_view('glyphs-all', pt, gen_gifs=True)
        logger.info("Sleeping for 10 secs. Giving Browserstack api a rest")
        time.sleep(10)

    diffbrowsers.diff_view('waterfall', gen_gifs=True)
    logger.info("Sleeping for 10 secs. Giving Browserstack api a rest")
    time.sleep(10)

    diffbrowsers.browsers = test_browsers['osx_browser']
    diffbrowsers.diff_view('glyphs-modified', gen_gifs=True)

    report = cli_reporter(diffbrowsers.stats)
    report_path = os.path.join(args.output_dir, 'report.txt')
    with open(report_path, 'w') as doc:
        doc.write(report)

    print(report)


if __name__ == '__main__':
    main()



"""
Visualize any differences found with fontdiffenator
"""
import argparse
from diffbrowsers.gfregression import GF_PRODUCTION_URL, VIEWS
from diffbrowsers.diffbrowsers import DiffBrowsers
from diffbrowsers.browsers import test_browsers
import os
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('fonts_after', nargs="+", help="Fonts after paths")
    before_group = parser.add_argument_group(title="Fonts before input")
    before_input_group = before_group.add_mutually_exclusive_group(required=True)
    before_input_group.add_argument('-fb', '--fonts-before', nargs="+",
                                    help="Fonts before paths")
    before_input_group.add_argument('-gf', '--from-googlefonts', action='store_true',
                               help="Diff against GoogleFonts instead of fonts_before")


    parser.add_argument('-u', '--gfr-url', default=GF_PRODUCTION_URL,
                        help="Url to GFR instance")
    parser.add_argument('-l', '--gfr-local', action="store_true", default=False)
    parser.add_argument('-o', '--output-dir', help="Directory for output images",
                               required=True)
    args = parser.parse_args()

    browsers_to_test = test_browsers['safari_latest']

    diffbrowsers = DiffBrowsers(gfr_instance_url=args.gfr_url,
                                gfr_is_local=args.gfr_local,
                                dst_dir=args.output_dir,
                                browsers=browsers_to_test)

    fonts_before = 'from-googlefonts' if args.from_googlefonts \
                    else args.fonts_before

    diffbrowsers.new_session(fonts_before, args.fonts_after)

    views_to_diff = diffbrowsers.gf_regression.info['diffs']
    logger.info("Following diffs have been found [%s]. Genning images." % ', '.join(views_to_diff))
    for view in views_to_diff:
        logger.info("Generating images for {}".format(view))
        if view not in VIEWS:
            logger.info("Skipping view {}".format(view))
        else:
            diffbrowsers.diff_view(view, pt=32)

    logger.info("Images saved to {}".format(args.output_dir))


if __name__ == '__main__':
    main()



"""
Compare VFs against static family on GF.

Test all glyphs at text sizes
"""
import argparse
import os
import logging
from diffbrowsers.gfregression import GF_PRODUCTION_URL, VIEWS
from diffbrowsers.diffbrowsers import DiffBrowsers
from diffbrowsers.browsers import test_browsers

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--fonts', nargs="+", required=True)
    parser.add_argument('-u', '--gfr-url', default=GF_PRODUCTION_URL,
                        help="Url to GFR instance")
    parser.add_argument('-l', '--gfr-local', action="store_true", default=False)
    parser.add_argument('-o', '--output-dir', help="Directory for output images",
                        required=True)
    args = parser.parse_args()

    browsers_to_test = test_browsers['vf_browsers']
    if not os.path.isdir(args.output_dir):
        os.mkdir(args.output_dir)

    diffbrowsers = DiffBrowsers(gfr_instance_url=args.gfr_url,
                                gfr_is_local=args.gfr_local,
                                dst_dir=args.output_dir,
                                browsers=browsers_to_test)

    diffbrowsers.new_session("from-googlefonts", args.fonts)
    logger.info("Generating waterfall diff")
    diffbrowsers.diff_view("waterfall")
    for pt_size in [13, 14, 15, 16]:
        logger.info("Generating images for glyphs_all at {}".format(pt_size))
        diffbrowsers.diff_view("glyphs_all", pt=pt_size)
    logger.info("Images saved to {}".format(args.output_dir))


if __name__ == '__main__':
    main()



from __future__ import division, absolute_import
import logging
import requests
import json


logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

GF_PRODUCTION_URL = 'http://www.gf-regression.com'

VIEWS = (
    'waterfall',
    'glyphs_all', 'glyphs_new', 'glyphs_missing', 'glyphs_modified',
    'marks_new', 'marks_missing', 'marks_modified',
    'mkmks_new', 'mkmks_missing', 'mkmks_modified',
    'kerns_new', 'kerns_missing', 'kerns_modified',
    'metrics_modified',
)


class UnknownGFRegressionViewError(Exception):
    def __init__(self):
        super(UnknownGFRegressionViewError, self).__init__(
        'View is not valid. Choose from [%s]' % ', '.join(VIEWS)
    )


class GFRegression:
    """Simple client for GF Regression"""
    def __init__(self, instance_url=GF_PRODUCTION_URL):
        self.instance_url = instance_url
        self._validate_instance(self.instance_url)
        self.info = {}

    def new_session(self, fonts_before, fonts_after):
        """Post fonts to GF Regression site using the api.

        If fonts_before == 'from-googlefonts', compare against fonts hosted
        on Google Fonts.

        If the fonts uploaded successfully, GF Regression will return a uuid.
        This can be used to form urls to view endpoints."""
        logger.info("Posting fonts to GF Regression")
        if fonts_before == 'from-googlefonts':
            url_upload = self.instance_url + '/api/upload/googlefonts'
            payload = [('fonts_after', open(f, 'rb')) for f in fonts_after]
        else:
            url_upload = self.instance_url + '/api/upload/user'
            payload = [('fonts_after', open(f, 'rb')) for f in fonts_after] + \
                      [('fonts_before', open(f, 'rb')) for f in fonts_before]
        request = requests.post(url_upload, files=payload)
        self.info = request.json()
        logger.info("Fonts have been uploaded, uuid: %s" % self.info['uuid'])

    def load_session(self, url):
        """Load fonts which were previously posted to GF Regression"""
        uuid = self._extract_uuid(url)
        self.info = self._session_info(uuid)

    def url(self, view, font_type, pt=None):
        """Return a url from a user's input params."""
        if view not in VIEWS:
            raise UnknownGFRegressionViewError()
        if not self.info['uuid']:
            raise Exception('No fonts uploaded or previous uuid defined')
        url = '%s/screenshot/%s/%s/%s' % (self.instance_url,
                                          self.info['uuid'], view, font_type)
        if pt:
            url = url + '/%s' % pt
        return url

    @staticmethod
    def _extract_uuid(url):
        """Extract a uuid4 subpath from a url.

        http://127.0.0.1:5000/compare/a3ec8a52-690d-4faf-b567-13a488125c62/fonts
        -->
        a3ec8a52-690d-4faf-b567-13a488125c62
        """
        segments = url.split('/')
        for idx, segment in enumerate(segments):
            dash_count = 0
            for char in segment:
                if char == '-':
                    dash_count += 1
            if dash_count == 4:
                return segments[idx]
        raise Exception('Url does not contain a valid uuid4')

    def _session_info(self, uuid):
        """Return info about the current session"""
        url = "%s/api/info/%s" % (self.instance_url, uuid)
        request = requests.get(url)
        if request.status_code != 200:
            raise Exception('url %s is invalid' % url)
        return request.json()

    def _validate_instance(self, url):
        """Confirm instance_url is a working instance of GFRegression"""
        request = requests.get(url)
        if 'Google Fonts Regression' not in request.text and \
           'Compare fonts' not in request.text:
            raise Exception(('instance_url %s is not an instance of '
                             'GF Regression' % url))



from __future__ import division, absolute_import
import logging
import browserstack_screenshots
import os
import time
from diffbrowsers.utils import download_file


logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

class ScreenShot(browserstack_screenshots.Screenshots):
    """Expansion for browserstack screenshots Lib. Adds ability to
    download files"""

    def take(self, url, dst_dir):
        """take a screenshot from a url and save it to the dst_dir"""
        self.config['url'] = url
        logger.info('Taking screenshot for url: %s' % url)
        generate_resp_json = self.generate_screenshots()
        job_id = generate_resp_json['job_id']

        logger.info('Browserstack is processing: '
                    'http://www.browserstack.com/screenshots/%s' % job_id)
        screenshots_json = self.get_screenshots(job_id)
        while screenshots_json == False: # keep refreshing until browerstack is done
            time.sleep(3)
            screenshots_json = self.get_screenshots(job_id)
        for screenshot in screenshots_json['screenshots']:
            filename = self._build_filename_from_browserstack_json(screenshot)
            base_image = os.path.join(dst_dir, filename)
            try:
                download_file(screenshot['image_url'], base_image)
            except:
                logger.info('Skipping {} BrowserStack timed out'.format(
                    screenshot['image_url'])
                )

    def _build_filename_from_browserstack_json(self, j):
        """Build useful filename for an image from the screenshot json
        metadata"""
        filename = ''
        device = j['device'] if j['device'] else 'Desktop'
        if j['state'] == 'done' and j['image_url']:
            detail = [device, j['os'], j['os_version'],
                      j['browser'], j['browser_version'], '.jpg']
            filename = '_'.join(item.replace(" ", "_") for item in detail if item)
        else:
            logger.info('screenshot timed out, ignoring this result')
        return filename



import shutil
import requests
import os
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


CONFIG_FILE = '~/.browserstack-api-config'


class NoBrowserStackAuthFile(Exception):
    def __init__(self):
        super(NoBrowserStackAuthFile, self).__init__(
            "~/.browserstack-api-config file is missing. See README.md"
    )


def load_browserstack_credentials():
    """Load the credentials to use Browserstack's screenshot api."""
    config = ConfigParser()
    config_filepath = os.path.expanduser(CONFIG_FILE)

    if os.path.isfile(config_filepath):
        config.read(config_filepath)
        credentials = config.items('Credentials')
        return credentials[0][1], credentials[1][1]
    return None

def cli_reporter(report):
    """Simple output for report dict

    input: {'view-name' [(platform, px_difference)]}

    output: str

    e.g

    >>> reporter(diffbrowsers.report)

    Regression Report:
    Fonts: [fonts]

    View: Glyphs-All
    WARNING: Desktop_Windows_7_chrome_50.0 is different by 100px,
    WARNING: Desktop_OS_X_Yosemite_safari_8.0 is different by 80px

    View: Waterfall
    PASSED: Desktop_Windows_7_chrome_50.0 is different is the same
    PASSED: Desktop_OS_X_Yosemite_safari_8.0 is different is the same

    TODO (M Foley) this needs more work
    """
    doc = []
    doc.append('Regression Report:\n\n')
    doc.append('Fonts: ["{}"]\n'.format('", "'.join(report['fonts'])))
    for view in report['views']:
        doc.append('\nView: {}\n'.format(view))
        for platform, px_diff in report['views'][view]:
            if px_diff != 0:
                doc.append('WARNING: {} is different by {} pixels\n'.format(
                    platform, px_diff
                ))
            else:
                doc.append('PASSED: {} is the same\n'.format(platform))
    return ''.join(doc)


def download_file(url, dst_path=None):
    """Download a file from a url. If no url is specified, store the file
    as a StringIO object"""
    try:
        request = requests.get(url, stream=True)
        if not dst_path:
            return StringIO(request.content)
        with open(dst_path, 'wb') as downloaded_file:
            shutil.copyfileobj(request.raw, downloaded_file)
    except requests.exceptions.MissingSchema:
        raise Exception("url {} is not a valid file".format(url))



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Browserdiff
~~~~~~~~~~~

Compare two sets of fonts for regressions

Caveats, script is incredibly slow due to the Browserstack api. Fonts are
matched by filenames.

See README.md for further info

Basic Usage:
gfdiffbrowsers new [fonts_after] -fb [fonts_before] -o ~/Desktop/font_img_dir

Compare against family hosted on Google Fonts:
gfdiffbrowsers new [fonts_after] -gf -o ~/Desktop/font_img_dir

Load a previous session:
gfdiffbrowsers load <url> -o ~/Desktop/font_img_dir
"""
from __future__ import print_function, division, absolute_import, unicode_literals
import argparse
import os

from diffbrowsers.diffbrowsers import DiffBrowsers
from diffbrowsers.browsers import test_browsers
from diffbrowsers.gfregression import GF_PRODUCTION_URL, VIEWS
from diffbrowsers.utils import cli_reporter
import logging

logging.basicConfig(level=logging.INFO)


def main():
    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument('-u', '--gfr-url', default=GF_PRODUCTION_URL,
                               help="Url to GFR instance")
    parent_parser.add_argument('-l', '--gfr-local', action="store_true", default=False,
                               help="Enable this if GFR is being run locally.")
    parent_parser.add_argument('-o', '--output-dir', help="Directory for output images",
                            required=True)
    parent_parser.add_argument('-pt', '--type-point-size',
                            help="In some views, users can control type sample size")
    parent_parser.add_argument('-b', '--browsers',
                        choices=list(test_browsers.keys()),
                        default='all_browsers',
                        help="Which set of browsers to test on")
    parent_parser.add_argument('-v', '--view', choices=VIEWS, default='waterfall')
    parent_parser.add_argument('-gif', '--output-gifs', action='store_true', default=True,
                        help="Output before and after gifs")

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest='command')

    upload_parser = subparsers.add_parser('new', parents=[parent_parser])
    upload_parser.add_argument('fonts_after', nargs="+", help="Fonts after paths")
    before_group = upload_parser.add_argument_group(title="Fonts before input")
    before_input_group = before_group.add_mutually_exclusive_group(required=True)
    before_input_group.add_argument('-fb', '--fonts-before', nargs="+",
                                  help="Fonts before paths")
    before_input_group.add_argument('-gf', '--from-googlefonts', action='store_true',
                               help="Diff against GoogleFonts instead of fonts_before")

    load_parser = subparsers.add_parser('load', parents=[parent_parser])
    load_parser.add_argument("url")

    args = parser.parse_args()

    browsers_to_test = test_browsers[args.browsers]

    diffbrowsers = DiffBrowsers(gfr_instance_url=args.gfr_url,
                                gfr_is_local=args.gfr_local,
                                dst_dir=args.output_dir,
                                browsers=browsers_to_test)

    if args.command == 'new':
        fonts_before = 'from-googlefonts' if args.from_googlefonts \
                       else args.fonts_before
        diffbrowsers.new_session(fonts_before, args.fonts_after)
    elif args.command == 'load':
        diffbrowsers.load_session(args.url)

    diffbrowsers.diff_view(args.view, args.type_point_size, args.output_gifs)

    report_path = os.path.join(args.output_dir, 'report.txt')
    with open(report_path, 'w') as doc:
        report = cli_reporter(diffbrowsers.stats)
        doc.write(report)
        print(report)


if __name__ == '__main__':
    main()



# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os
from setuptools import setup, find_packages, Command
from distutils import log

setup(
    name='gfdiffbrowsers',
    version='0.1.3',
    author="Marc Foley",
    author_email="marc@mfoley.uk",
    description="Diff two sets of fonts in different browsers",
    url="https://github.com/googlefonts/diffbrowsers",
    license="Apache Software License 2.0",
    package_dir={"": "Lib"},
    packages=find_packages("Lib"),
    entry_points={
        "console_scripts": [
            "gfdiffbrowsers = diffbrowsers.__main__:main"
        ],
    },
    scripts=[
        os.path.join('bin', 'test_gf_vf.py'),
        os.path.join('bin', 'test_gf_autohint.py'),
        os.path.join('bin', 'test_gf_exhaustive.py'),
        os.path.join('bin', 'viz_diffenator.py')
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        "Environment :: Console",
        "Environment :: Other Environment",
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Multimedia :: Graphics',
        'Topic :: Multimedia :: Graphics :: Graphics Conversion',
        'Topic :: Multimedia :: Graphics :: Editors :: Vector-Based',
    ],
    install_requires=[
        "idna==2.6",
        "nose==1.3.7",
        "olefile==0.44",
        "Pillow",
        "pybrowserstack-screenshots==0.1",
        "requests==2.18.4",
        "simplejson==3.12.0",
        "urllib3==1.22",
    ],
)



from __future__ import print_function, division, absolute_import
from PIL import Image, ImageChops
import requests
import os
from glob import glob
from ntpath import basename
import json
import time
import shutil
import logging

from diffbrowsers.gfregression import GFRegression, GF_PRODUCTION_URL
from diffbrowsers.browsers import test_browsers
from diffbrowsers.screenshot import ScreenShot
from diffbrowsers.utils import (
    load_browserstack_credentials,
    NoBrowserStackAuthFile
)

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

class DiffBrowsers(object):
    """Class to control GF Regression and Browser Stack api."""
    def __init__(self,
                 auth=None,
                 dst_dir=None,
                 browsers=test_browsers['all_browsers'],
                 gfr_instance_url=GF_PRODUCTION_URL,
                 gfr_is_local=False):

        if not auth:
            auth = load_browserstack_credentials()
            if not auth:
                raise NoBrowserStackAuthFile

        if gfr_instance_url.endswith('/'):
            gfr_instance_url = gfr_instance_url[:-1]

        self.gf_regression = GFRegression(
            instance_url=gfr_instance_url
        )
        self.browserstack_settings = browsers
        if gfr_is_local:
            self.browserstack_settings['local'] = True
        self.screenshot = ScreenShot(auth=auth, config=self.browserstack_settings)

        self.dst_dir = dst_dir if dst_dir else 'out'
        self.stats = {'views': {},
                      'fonts': []}
        self._mkdir(self.dst_dir)

    def new_session(self, fonts_before, fonts_after):
        """Upload fonts to gfregression"""
        self.gf_regression.new_session(fonts_before, fonts_after)
        logger.info("Posting fonts to GF Regression. Be patient.")
        self.stats['fonts'] = self.gf_regression.info['fonts']

    def load_session(self, url):
        """Load a previous gf regression session"""
        self.gf_regression.load_session(url)
        self.stats['fonts'] = self.gf_regression.info['fonts']

    def diff_view(self, screenshot_view, pt=None, gen_gifs=True):
        """Return before and after images from a GF Regression view.

        Use PIL to calculate the amount of different pixels and save
        the images."""
        if not self.gf_regression.info['uuid']:
            raise Exception("Cannot make diff. Upload or load fonts first")
        view_dir = '{}_{}pt'.format(screenshot_view, pt) if pt \
                   else screenshot_view
        view_path = os.path.join(self.dst_dir, view_dir)
        self._mkdir(view_path, overwrite=True)

        logger.info('Generating {} before images'.format(screenshot_view))
        before_url = self.gf_regression.url(screenshot_view, 'before', pt)
        before_path = os.path.join(view_path, 'before')
        self._mkdir(before_path, overwrite=True)
        self.screenshot.take(before_url, before_path)

        logger.info('Generating {} after images'.format(screenshot_view))
        after_url = self.gf_regression.url(screenshot_view, 'after', pt)
        after_path = os.path.join(view_path, 'after')
        self._mkdir(after_path, overwrite=True)
        self.screenshot.take(after_url, after_path)

        diff_dir = os.path.join(view_path, 'diff')
        self._mkdir(diff_dir, overwrite=True)

        comparison = self._compare_images(before_path, after_path, diff_dir)
        r_view = '{}_{}pt'.format(screenshot_view, pt) if pt else screenshot_view
        self.stats['views'][r_view] = comparison

        if gen_gifs:
            logger.info('Generating {} gifs'.format(screenshot_view))
            gif_dir = os.path.join(view_path, 'gifs')
            self._mkdir(gif_dir, overwrite=True)
            self._gen_gifs(before_path, after_path, gif_dir)
        return comparison

    def _mkdir(self, path, overwrite=False):
        """Create a directory, if overwrite enabled rm -rf the dir"""
        if not os.path.isdir(path):
            os.mkdir(path)
        if overwrite:
            shutil.rmtree(path)
            os.mkdir(path)

    def _gen_gifs(self, dir1, dir2, out_dir):
        shared_imgs = self._matched_filenames_in_dirs(dir1, dir2, 'jpg')
        for img in shared_imgs:
            gif_filename = img[:-4] + '.gif'
            dir1_img_path = os.path.join(dir1, img)
            dir2_img_path = os.path.join(dir2, img)
            if not self._valid_imgs([dir1_img_path, dir2_img_path]):
                logger.warning(("Skipping {}. Before/after images are "
                             "corrupt").format(gif_filename))
                continue
            with Image.open(dir1_img_path) as dir1_img, \
                 Image.open(dir2_img_path) as dir2_img:
                dir1_img.save(
                    os.path.join(out_dir, gif_filename),
                    save_all=True,
                    append_images=[dir2_img],
                    loop=10000,
                    duration=1000
                )

    def _matched_filenames_in_dirs(self, dir1, dir2, ext):
        """find matching filenames in two different dirs which have a specific
        extension"""
        dir1_items = {basename(n): n for n in glob('%s/*.%s' % (dir1, ext))}
        dir2_items = {basename(n): n for n in glob('%s/*.%s' % (dir2, ext))}
        return set(dir1_items) & set(dir2_items)

    def _compare_images(self, dir1, dir2, diff_dir):
        """Compare two folders of images against each other."""
        comparisons = []

        shared_imgs = self._matched_filenames_in_dirs(dir1, dir2, 'jpg')
        for img in shared_imgs:
            dir1_img_path = os.path.join(dir1, img)
            dir2_img_path = os.path.join(dir2, img)
            diff_img_path = os.path.join(diff_dir, img)
            if not self._valid_imgs([dir1_img_path, dir2_img_path]):
                logger.warning(("Skipping {}. Before/after images are "
                             "corrupt").format(diff_img_path))
                continue
            with Image.open(dir1_img_path) as dir1_img, \
                Image.open(dir2_img_path) as dir2_img:
                comparison = compare_image(dir1_img, dir2_img, diff_img_path)
            comparisons.append((img, comparison))
        return comparisons

    def update_browsers(self, browsers):
        self.screenshot.config['browsers'] = browsers['browsers']

    def _valid_imgs(self, imgs_paths):
        for img_path in imgs_paths:
            if os.path.getsize(img_path) == 0:
                return False
        return True


def compare_image(img1, img2, out_img=None,
                  ignore_first_px_rows=200, ignore_right_px_cols=40):
    """Compare two images and return the amount of different pixels.

    ignore_first_px_rows param will ignore the first n pixel rows. This is
    useful if the images contain text which shouldn't be diffed and may
    change such as a header.

    ignore_right_px_cols param will ignore the last n pixel columns. GF Regression
    shows before and after labels in the right hand margin. We don't want these
    labels to be included in the pixel count."""
    img_diff = ImageChops.difference(img1, img2)

    pixels = list(img_diff.getdata())
    width, height = img_diff.size
    pixels = [pixels[i * width:(i + 1) * width] for i in range(height)]

    px_diff = 0
    for line in pixels[ignore_first_px_rows:]:
        for px in line[:-ignore_right_px_cols]:
            # ignore image alpha channel if exists
            r, g, b = px[:3]
            if r != 0 or g != 0 or b != 0:
                px_diff += 1
    if out_img:
        img_diff_rgb = img_diff.convert('RGB')
        img_diff_rgb.save(out_img[:-4] + '.png')
    img_diff.close()
    return px_diff
